export P=pyqt5
export V=5.11.3
export B=1
export MAINTAINER=JuergenFischer

export PYTHON=Python37

savelog() {
	local f=$1
	rm -f $f.9
	for i in $(seq 8 -1 0); do [ -f "$f.$i" ] && mv $f.$i $f.$(( i + 1 )); done
	[ -f $f ] && mv $f $f.0
}

savelog package.log
exec > >(tee package.log) 2>&1
date

set -e

[ -f PyQt5_gpl-$V.tar.gz ] || wget https://sourceforge.net/projects/pyqt/files/PyQt5/PyQt-$V/PyQt5_gpl-$V.tar.gz
if ! [ -f ../configure.py ]; then
	tar -C .. -xzf PyQt5_gpl-$V.tar.gz --xform s,PyQt5_gpl-$V,.,
	patch -p1 -d .. <public_sip.diff
fi

if ! [ -d "$OSGEO4W_REP" ]; then
        echo no repository
        exit 1
fi

fetchenv() {
        local IFS
        IFS=
        batch=$1
        shift

        if ! [ -f "$batch" ]; then
                echo "$batch not found"
                return 1
        fi

        srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
        dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
        diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
        envdiff=$(mktemp /tmp/envdiff.XXXXXXXXXX)
        cmd /c set >$srcenv
        cmd /c call $(cygpath -w $batch) "$@" \>nul 2\>nul \& set >$dstenv
        /usr/bin/cat <<EOF >$envdiff
/^+++/d;
/^[^+]/d;
s/^+//;
# s/\\\\/\\\\\\\\/g;
/^PATH=/ {
        s/\\\\/\\//g;
        s#\\([=;]\\)\\([a-zA-Z]\\):#\\1/cygdrive/\\2#g;
        s/;/:/g;
}
s/;/\\\\;/g;
s/^/export /;
s/=/='/;
s/\\\\r//g;
s/\r$/'/;
EOF
        /usr/bin/diff -u $srcenv $dstenv | /usr/bin/sed -f $envdiff >$diffenv
        . $diffenv
        PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
        rm -f $srcenv $dstenv $diffenv $envdiff
}

for ARCH in x86 x86_64; do
	export ARCH

	(
		case "$ARCH" in
		x86)
        		export OSGEO4W_ROOT=/c/OSGeo4W
        		export VCARCH=x86
			;;

		x86_64)
			export OSGEO4W_ROOT=/c/OSGeo4W64
	        	export VCARCH=amd64
			;;

		*)
        		echo invalid arch
        		exit 1
			;;

		esac

		export R=$OSGEO4W_REP/$ARCH/release/python3/$P
		mkdir -p $R

		export INSTDIR=install-$ARCH

		fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat
		fetchenv $OSGEO4W_ROOT/bin/py3_env.bat
		fetchenv $OSGEO4W_ROOT/bin/qt5_env.bat

		fetchenv "/c/Program Files (x86)/Microsoft Visual Studio 14.0/VC/vcvarsall.bat" $VCARCH
		export PATH=$PATH:/bin

		export INCLUDE="$(cygpath -m $OSGEO4W_ROOT/apps/$PYTHON/include);$INCLUDE"
		#export LIB="$LIB;$(cygpath -m $OSGEO4W_ROOT/lib)"

		cat <<EOF >$R/setup.hint
sdesc: "Python bindings for Qt5."
ldesc: "Python bindings for Qt5."
maintainer: $MAINTAINER
category: Libs
requires: python3-core qt5-libs
EOF

		cd ..
		export DESTDIR=$PWD/osgeo4w/$INSTDIR
		rm -rf $DESTDIR
		mkdir -p $DESTDIR

		rm -rf release
		mkdir -p release
		for i in release .; do cp $OSGEO4W_ROOT/bin/qt.conf $i; done

		python configure.py \
 			--verbose \
			--confirm-license \
			--destdir "$(cygpath -m $DESTDIR/apps/$PYTHON/lib/site-packages)" \
			--sipdir "$(cygpath -m $DESTDIR/apps/$PYTHON/sip/PyQt5)" \
			--designer-plugindir "$(cygpath -m $DESTDIR/apps/Qt5/plugins/designer)" \
			--stubsdir "$(cygpath -m $DESTDIR/apps/$PYTHON/lib/site-packages/PyQt5)" \
			--no-qml-plugin \
			--disable QtNfc \
			--qsci-api-destdir "$(cygpath -m $DESTDIR/apps/Qt5/qsci)" \
			--bindir "$(cygpath -m $DESTDIR/apps/$PYTHON/Scripts)"

		nmake clean
		nmake
		nmake install

		cd osgeo4w
		mkdir -p $INSTDIR/etc/postinstall
		mkdir -p $INSTDIR/etc/preremove

		cat <<EOF >$INSTDIR/etc/postinstall/$P.bat
set OSGEO4W_ROOT=%OSGEO4W_ROOT:\\=\\\\%
textreplace -std -t apps\\$PYTHON\\Scripts\\pyuic5.bat
textreplace -std -t apps\\$PYTHON\\Scripts\\pyrcc5.bat
textreplace -std -t apps\\$PYTHON\\Scripts\\pylupdate5.bat
EOF

		cat <<EOF >$INSTDIR/etc/preremove/$P.bat
del apps\\$PYTHON\\Scripts\\pyuic5.bat
del apps\\$PYTHON\\Scripts\\pyrcc5.bat
del apps\\$PYTHON\\Scripts\\pylupdate5.bat
EOF

		export O4W_ROOT="$(cygpath --windows --short-name $OSGEO4W_ROOT | sed -e 's/\\/\\\\/g')"

		for i in pyuic5.bat pyrcc5.bat pylupdate5.bat; do
			sed -e "s/$O4W_ROOT/@osgeo4w@/g" $INSTDIR/apps/$PYTHON/Scripts/$i >$INSTDIR/apps/$PYTHON/Scripts/$i.tmpl
			rm $INSTDIR/apps/$PYTHON/Scripts/$i
		done

		cp ../LICENSE $R\$P-$V-$B.txt
		/usr/bin/tar -C $INSTDIR -cjf $R/$P-$V-$B.tar.bz2 apps etc
		/usr/bin/tar -C .. -cjf $R/$P-$V-$B-src.tar.bz2 osgeo4w/package.sh osgeo4w/public_sip.diff
	)
done

date
